# Script to construct our main impact projections under LGM climate change
#modified from from the replication data provided by Burke et al. (2015) 
#at https://purl.stanford.edu/wb587wt4560  for a LGM temperature change and using the prefered model of Newell et al. (2018) 
#(temperature impact on GDP level and not on growth as in Burke et al. (2015))

# Main steps in the script:
#   1. Assembles the necessary scenarios on future population and income growth.  These come from the
#       Shared Socioeconomic Pathways (SSPs), as well as from a "baseline" scenario that fixes future growth rates
#       in absence of climate change at historical rates as.  These scenarios are saved as lists.
#   2. Reads in projected country-level temperature change for the LGM scenario.  These are calculated in the GetTemperatureChange_LGM05.R script
#   3. Projects changes in per capita GDP  
# The main output is country- and year-specific per capita GDP projections to 2100, with and without LGM climate change, 
#     These are written out. 


rm(list = ls())

require(maptools)
require(fields)
require(classInt)
require(plotrix)
require(data.table)
require(dplyr)
require(ncdf)
require(raster)
library(maps)
library(wq)
library(ggplot2)
library(xtable)
library(reshape2)
"%&%"<-function(x,y)paste(x,y,sep="")  #define a function for easy string pasting

#setwd("myFolder")

dir.create("data/output/projectionOutput")

# FIRST DEFINE FUNCTION TO INTERPOLATE SSP POPULATION AND GROWTH DATASETS.  THESE COME AS 5-YR PROJECTIONS. we want opbs for each year for each projection, so linearly interpolate between 5-yr estimates
#   writing this as a function that spits out a data frame, where the first three columns gives the scenario and country, and the rest give the projections by year
#   growth projections only go through 2095, so repeating the 2095 value for 2096-2099
ipolate <- function(mat) {
  mat1 <- array(dim=c(dim(mat)[1],length(yrs)))
  ys <- as.numeric(unlist(strsplit(names(mat),"X")))
  est <- seq(2010,2100,5)  #the 5yr estimates in the SSP dataset
  for (i in 1:length(yrs)) {
    y = yrs[i]
    if ("X"%&%y %in% names(pop) == T) {  #if the year falls on the 5-yr interval, use their point estimate. otherwise interpolate between nearest endpoints
      mat1[,i]  <- as.numeric(mat[,which(names(mat)=="X"%&%y)])
    } else {
      z <- y-est
      yl = est[which(z==min(z[z>0]))]  #the 5-year endpoint lower than the year
      y5 = yl+5  #the next endpoint
      el <- as.numeric(mat[,which(names(mat)=="X"%&%yl)])  #values at lower endpoint
      eu <- as.numeric(mat[,which(names(mat)=="X"%&%y5)]) #values at upper endpoint
      if (y > max(ys,na.rm=T)) {  mat1[,i] <- el   #this is to account for growth projections ending in 2095  
      }  else { mat1[,i] <- el + (eu-el)*(y-yl)/5 }
    }
  } 
  mat1 <- data.frame(mat[,1:3],mat1)
  names(mat1)[4:dim(mat1)[2]] <- yrs
  levels(mat1$Region)[levels(mat1$Region)=="COD"] <- "ZAR"  #our code for the DRC
  levels(mat1$Region)[levels(mat1$Region)=="ROU"] <- "ROM"  #our code for Romania  
  return(mat1)
}


dta <- read.csv("output/mainDataset.csv") 
gdpCap = dta$TotGDP/dta$Pop
dta <- data.frame(dta,gdpCap)
mt <- dta %>%   #the following few lines gets the average temperature in each country for the years we want, using dplyr
  filter(year>=1980 & is.na(UDel_temp_popweight)==F & is.na(growthWDI)==F) %>% 
  group_by(iso) %>% 
  summarize(meantemp = mean(UDel_temp_popweight,na.rm=T), basegrowth = mean(growthWDI, na.rm=T), gdpCap = mean(gdpCap,na.rm=T))

mt <- as.data.frame(mt) 
yrs <- 2010:2099


# We will also run our own scenario, where future growth rates are just set at their historical average for each country, 
# and pop projections come from UN
# Need to get these in same shape as SSP above so they can easily be fed into the calculations below

# First process the UN population data
pop <- read.csv("input/populationData/WPP2012_DB02_POPULATIONS_ANNUAL.csv")
# reshape to wide, just keeping yrs 2010 and after
pop <- pop[pop$Time%in%2010:2100,]  
pop <- pop[,c(1,5,9)]
pop <- reshape(pop,v.names="PopTotal",timevar="Time",idvar="LocID",direction="wide")
pop <- pop[pop$LocID<900,]
#this file matches location ID in UN database to our ISO codes, with excpetion of DRC and ROM, as in the SSP
cod <- read.csv("input/populationData/WPP2012_F01_LOCATIONS.csv")  
cod <- cod[cod$LocID<900,]
iso <- as.character(cod$ISO3_Code)
iso[iso=="COD"] <- "ZAR"
iso[iso=="ROU"] <- "ROM"
LocID <- cod$LocID
cod <- data.frame(LocID, iso)
cod <- cod[iso!="GRL",]  #dropping greenland because not in SSP
poploc <- merge(pop,cod,by="LocID")
popprojb <- merge(mt,poploc,by="iso")  #165 countries which we have both baseline data and pop projections for
popprojb <- popprojb[,names(popprojb)!="LocID"]
names(popprojb)[5:dim(popprojb)[2]] <- yrs
popprojb[5:dim(popprojb)[2]] <- popprojb[5:dim(popprojb)[2]]/1000 #units will be in millions to match SSP
basegrowth <- matrix(rep(popprojb$basegrowth,length(yrs)),ncol=length(yrs),byrow=F) #same growth rate for every year, which is the baseline growth rate
colnames(basegrowth) <- yrs
basegrowth <- cbind(popprojb[,1:4],basegrowth)

popProjections <- NULL  #initialize list that we will will with population projections for each scenario
growthProjections <- NULL   #same for growth
popProjections[[1]] <- popprojb[]  
growthProjections[[1]] <- basegrowth


pop <- read.csv("input/SSP/SSP_PopulationProjections.csv")
levels(pop$Scenario)[levels(pop$Scenario)=="SSP4d_v9_130115"] <- "SSP4_v9_130115"  #renaming one of the scenarios slightly so the loop works
growth <- read.csv("input/SSP/SSP_GrowthProjections.csv")
pop1 <- ipolate(pop)  #warning here is just from stringsplit function
growth1 <- ipolate(growth)
growth1[,names(growth1)%in%yrs] = growth1[,names(growth1)%in%yrs]/100

#   First we merge countries in historical database with the growth and pop projections from SSP,
#restricted to the scenario we want
# we are using growth projections from OECD, which are the only ones with data for every country; 
#population projections are from IIASA
popSSP <- merge(mt,pop1,by.x="iso",by.y="Region")  #merge our data and SSP for population
# length(unique(popproj$iso))  #165 countries that we can match in our data to SSP data
growthSSP <- merge(mt,growth1,by.x="iso",by.y="Region")

for (scen in 1:5) {  #now add each scenario to the list
  # projections for economic growth - using OECD, because they have projections for every country
  pgrow <- growthSSP$Model=="OECD Env-Growth" & growthSSP$Scenario=="SSP"%&%scen%&%"_v9_130325"
  growthProjections[[scen+1]] <- growthSSP[pgrow,]
  
  # population projections from IIASA
  ppop <- popSSP$Scenario=="SSP"%&%scen%&%"_v9_130115"
  popProjections[[scen+1]] <- popSSP[ppop,]
}


# SAVE THIS SCENARIO DATA TO BE USED IN CONSTRUCTION OF DAMAGE FUNCTION
save(popProjections,file="output/projectionOutput/popProjections.Rdata")
save(growthProjections,file="output/projectionOutput/growthProjections.Rdata")


# Finally, read in projections of future temperature change, generated by the getTemperatureChange.R script
#   These are population-weighted country level projections averaged across all CMIP5 models
Tchg <- read.csv("input/CCprojections/CountryTempChange_LGM05.csv")
Tchg <- merge(popProjections[[1]][,1:3],Tchg,by.x="iso",by.y="GMI_CNTRY")
Tchg <- Tchg[Tchg$CNTRY_NAME%in%c("West Bank","Gaza Strip","Bouvet Island")==F,]
Tchg <- Tchg$Tdiff_w # just keep the vector of temperature changes, since sort order is now correct (#on perd l'indicateur pays !!!)


#####################################################################################################################
#  NOW WE ARE GOING TO RUN FUTURE PROJECTIONS OF IMPACTS BASED ON DIFFERENT HISTORICAL MODELS,
#     1. pooled model, zero lag
# 
#  	Calculating impacts relative to base period of 1980-2010; calculating each growth/pop scenario for each model
#   Writing results for each model out, so we can easily make plots of multiple scenarios if desired.  
#   For each we write out:  
#       (i) country specific GDP/cap for climate change and no climate change scenarios
#       (ii) global average GDP/cap, and total global GDP, for climate change and no climate change scenarios 
#####################################################################################################################

# set temperature change for all runs
dtm <- Tchg   #the country-specific changes
scens <- c("base","SSP"%&%1:5)
ccd <- dtm/length(yrs)  #rate of increase in temperature per year.  

####################################################################################################
# POOLED MODEL WITH NO LAGS
####################################################################################################


# now loop over SSP scenarios and our own scenario. just doing baseline, SSP3, SSP5 for now, since those appear to be the relevant ones
for (scen in c(1,4,6)) {
  
  growthproj <- growthProjections[[scen]]  #select growth projections
  popproj <- popProjections[[scen]]  # select population projections
  
  basegdp = popproj$gdpCap  #baseline GDP/cap
  temp <- popproj$meantemp  #baseline temperature.  
  
  GDPcapCC = GDPcapNoCC = array(dim=c(dim(growthproj)[1],length(yrs)))  #array to fill with GDP/cap for each country
  
  dimnames(GDPcapCC) <- dimnames(GDPcapNoCC) <- list(growthproj[,1],yrs)
  
  GDPcapCC[,1] = GDPcapNoCC[,1] = basegdp  #initialize with baseline per cap GDP
  tots = array(dim=c(length(yrs),4))  #array to hold average global per cap GDP and total global GDP across scenarios, with and without climate change
  dimnames(tots) <- list(yrs,c("avgGDPcapCC","avgGDPcapNoCC","TotGDPCC","TotGDPNoCC"))
  
  #coefficient values from Newell et al. (2018) 
  beta1 = 0.008141
  beta2 = -0.000314
  
  
  for (i in 2:length(yrs)) {
    j = i - 1
    y = yrs[i]
    basegrowth <- growthproj[,which(names(growthproj)==y)]  #growth rate without climate change
    GDPcapNoCC[,i] = GDPcapNoCC[,j]*(1+basegrowth)  #last year's per cap GDP times this years growth rate, as projected by scenario (basegrowth est le taux de croissance par scnario)
    newtemp = temp+j*ccd
    #we cap temperature at 30C and -5C
    newtemp=replace(newtemp,newtemp>30,30) 
    newtemp=replace(newtemp,newtemp<(-5),-5)
    
    GDPcapCC[,i] = GDPcapNoCC[,i]*exp(beta1*(newtemp-temp)+beta2*(newtemp*newtemp-temp*temp)) 
    
    #now calculate global average per cap GDP, weighting by population
    wt = popproj[,which(names(popproj)==y)]  #population weights 
    tots[i,1] <- round(weighted.mean(GDPcapCC[,i],wt),3)  # per cap GDP, with climate change
    tots[i,2] <- round(weighted.mean(GDPcapNoCC[,i],wt),3)  # per cap GDP, no climate change
    #total GDP with and without climate change. multiplying by 1e6 because population is in millions
    tots[i,3] <- sum(GDPcapCC[,i]*wt*1e6)  #with climate change
    tots[i,4] <- sum(GDPcapNoCC[,i]*wt*1e6) #without climate change
  }
  
  #write out scenario specific results
  GDPcapCC <- round(GDPcapCC,3) #round to nearest dollar
  GDPcapNoCC <- round(GDPcapNoCC,3)  #ditto
  save(GDPcapCC,file="output/projectionOutput/LGM_Level_GDPcapCC_pooled_"%&%scens[scen]%&%".Rdata")
  save(GDPcapNoCC,file="output/projectionOutput/LGM_Level_GDPcapNoCC_pooled_"%&%scens[scen]%&%".Rdata")
  save(tots,file="output/projectionOutput/LGM_Level_GlobalChanges_pooled_"%&%scens[scen]%&%".Rdata")
  GDPcapCC <- GDPcapNoCC <- NULL
  print(scen)
  
}

#world's GDP change
GDP_change = (tots[,3]-tots[,4])/tots[,4]*100


